<?php

namespace Drupal\entity_echo;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Messenger\Messenger;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\layout_builder\Section;

/**
 * Service for handling entity cloning and exporting operations.
 */
class EntityCloner {
  use StringTranslationTrait;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Drupal messenger service.
   *
   * @var \Drupal\Core\Messenger\Messenger
   */
  protected $messenger;

  /**
   * Constructs an EntityCloner object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Messenger\Messenger $messenger_svc
   *   Drupal messenger service.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    Messenger $messenger_svc,
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->messenger = $messenger_svc;
  }

  /**
   * Clones an entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity to clone.
   *
   * @return \Drupal\Core\Entity\EntityInterface
   *   The cloned entity.
   */
  public function cloneEntity(EntityInterface $entity) {
    // Exclude unique fields from Source node.
    $entity_type = $entity->getEntityTypeId();
    if ($entity_type === 'node') {
      // Nodes need vid unset.
      $exclude_fields = ['nid', 'revision_id', 'vid', 'uuid', 'path'];
    }

    if ($entity_type === 'taxonomy_term') {
      // Don't unset vid on terms, it's required.
      $exclude_fields = ['tid', 'revision_id', 'uuid', 'path'];
    }

    foreach ($exclude_fields as $field) {
      if ($entity->hasField($field)) {
        $entity->set($field, NULL);
      }
    }

    $cloned_entity = $entity->createDuplicate();
    $entity_type = $entity->getEntityTypeId();
    // Prepend 'Clone: ' to the newly created entity.
    if ($entity_type === 'node') {
      $title = $cloned_entity->getTitle();
      $cloned_entity->setTitle('Clone: ' . $title);
      // Only node has created time (strange).
      $cloned_entity->setCreatedTime(time());
    }
    if ($entity_type === 'taxonomy_term') {
      $title = $cloned_entity->getName();
      $cloned_entity->setName('Clone: ' . $title);
    }

    $cloned_entity->setChangedTime(time());

    $cloned_entity->save();

    $link = $cloned_entity->toLink($cloned_entity->label())->toString();
    $this->messenger->addStatus($this->t('Entity import successful: %link.', ['%link' => $link]));

    return $cloned_entity;
  }

  /**
   * Exports an entity to JSON.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity to export.
   *
   * @return string
   *   The JSON string representation of the entity.
   *
   * @throws \Exception
   *   If the entity cannot be exported.
   */
  public function exportEntity(EntityInterface $entity) {
    try {
      $entity_type = $entity->getEntityTypeId();
      $data = [
        'entity_type' => $entity_type,
        'type' => $entity->bundle(),
        'bundle' => $entity->bundle(),
        'langcode' => $entity->language()->getId(),
        'status' => $entity->isPublished() ? 1 : 0,
      ];

      if ($entity_type === 'node') {
        $data['uid'] = $entity->getOwnerId();
      }

      if ($entity_type === 'taxonomy_term') {
        $data['uid'] = $entity->getRevisionUser();
      }

      // Export fields.
      foreach ($entity->getFields() as $field_name => $field) {
        if ($field_name !== 'layout_builder__layout') {
          $data[$field_name] = $field->getValue();
        }
      }

      // Handle Layout Builder data.
      if ($entity->hasField('layout_builder__layout')) {
        $layout_data = [];
        foreach ($entity->get('layout_builder__layout')->getSections() as $section) {
          $layout_data[] = [
            'section' => $section->toArray(),
          ];
        }
        $data['layout_builder__layout'] = $layout_data;
      }

      return json_encode($data, JSON_PRETTY_PRINT | JSON_THROW_ON_ERROR);
    }
    catch (\Exception $e) {
      $this->loggerFactory->get('entity_echo')->error('Failed to export entity %id: %error', [
        '%id' => $entity->id(),
        '%error' => $e->getMessage(),
      ]);
      $this->messenger->addStatus($this->t('Failed to export entity'));

      throw $e;
    }
  }

  /**
   * Imports an entity from JSON.
   *
   * @param string $json
   *   The JSON string containing entity data.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   The imported entity, or NULL on failure.
   */
  public function importEntity($json) {
    try {
      $data = json_decode($json, TRUE, 512, JSON_THROW_ON_ERROR);

      // Init variable for creating a stub node.
      $entity_values = [];

      if (empty($data['type'])) {
        throw new \Exception('Invalid JSON: Missing entity bundle type.');
      }

      if (empty($data['entity_type'])) {
        throw new \Exception('Missing Entity Type (node, taxonomy_term.');
      }

      // If the entity exists, update it.
      if (!empty($data['uuid'])) {
        $entity_values['uuid'] = $data['uuid'][0]['value'];
        $existing = $this->entityTypeManager
          ->getStorage($data['entity_type'])
          ->loadByProperties(['uuid' => $data['uuid'][0]['value']]);
        if ($existing) {
          $entity = reset($existing);
          // Update the changed time when importing an existing node.
          $entity->setChangedTime(time());
        }
      }

      // Exclude fields that should not be imported.
      if (!empty($data['nid'])) {
        // Nodes need vid unset.
        $exclude_fields = ['nid', 'revision_id', 'vid', 'changed'];
      }

      if (!empty($data['tid'])) {
        // Don't unset vid on terms, it's required.
        $exclude_fields = ['tid', 'revision_id', 'changed'];
      }

      // We want to keep the uuid if the node already exists.
      if (empty($existing)) {
        $exclude_fields[] = 'uuid';
        $entity_values['vid'] = $data['vid'];
      }

      // Remove excluded fields from the data array.
      foreach ($exclude_fields as $field) {
        if (isset($data[$field])) {
          unset($data[$field]);
        }
      }

      // Set the rest of the stub node values.
      $entity_values['type'] = $data['type'];
      $entity_values['langcode'] = $data['langcode'] ?? 'en';
      $entity_values['status'] = $data['status'] ?? 1;
      $entity_values['uid'] = $data['uid'] ?? 1;

      if (!isset($entity)) {
        $entity = $this->entityTypeManager->getStorage($data['entity_type'])->create($entity_values);
      }

      // Set fields.
      foreach ($data as $field_name => $field_value) {
        if ($entity->hasField($field_name) && $field_name !== 'layout_builder__layout') {
          $entity->set($field_name, $field_value);
        }
      }

      // Handle Layout Builder data and inline blocks.
      if (!empty($data['layout_builder__layout']) && $entity->hasField('layout_builder__layout')) {
        $sections = [];
        foreach ($data['layout_builder__layout'] as $section_data) {
          if (!empty($section_data['section'])) {
            $section_array = $section_data['section'];
            // Update components with inline blocks.
            if (!empty($section_array['components'])) {
              foreach ($section_array['components'] as &$component) {
                if (!empty($component['configuration']['id']) && strpos($component['configuration']['id'], 'inline_block:') === 0) {
                  $block_revision_id = $this->ensureInlineBlock($component['configuration']);
                  if ($block_revision_id) {
                    $component['configuration']['block_revision_id'] = $block_revision_id;
                  }
                }
              }
            }
            $section = Section::fromArray($section_array);
            $sections[] = $section;
          }
        }
        $entity->get('layout_builder__layout')->setValue($sections);
      }

      $entity->save();
      $link = $entity->toLink($entity->label())->toString();
      if (!empty($existing)) {
        $this->messenger->addStatus($this->t('UPDATED Existing Entity: %link.', ['%link' => $link]));
      }
      else {
        $this->messenger->addStatus($this->t('Entity import successful: %link.', ['%link' => $link]));
      }

      return $entity;
    }
    catch (\Exception $e) {
      $this->messenger->addStatus($this->t('Failed to import entity: %error', ['%error' => $e->getMessage()]));
      return NULL;
    }
  }

  /**
   * Ensures an inline block exists or creates it.
   *
   * @param array $config
   *   The component configuration.
   *
   * @return int|null
   *   The block revision ID, or NULL on failure.
   */
  protected function ensureInlineBlock(array $config) {
    $block_type = explode(':', $config['id'])[1] ?? NULL;
    $block_uuid = $config['block_uuid'] ?? NULL;

    if (!$block_type || !$block_uuid) {
      $this->messenger->addStatus($this->t('Invalid block configuration.'));
      return NULL;
    }

    $block_storage = $this->entityTypeManager->getStorage('block_content');
    $blocks = $block_storage->loadByProperties(['uuid' => $block_uuid]);
    $block = reset($blocks);

    if (!$block) {
      $block = $block_storage->create([
        'type' => $block_type,
        'uuid' => $block_uuid,
        'info' => $config['label'] ?? 'Imported block',
      ]);
      $block->save();
    }

    return $block->getRevisionId();
  }

}
