<?php

namespace Drupal\entity_echo\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Response;
use Drupal\entity_echo\EntityCloner;

/**
 * Form to export a taxonomy term as JSON.
 */
class EntityExportTaxonomyForm extends FormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity cloner service.
   *
   * @var \Drupal\entity_echo\EntityCloner
   */
  protected $entityCloner;

  /**
   * Constructs an EntityExportTaxonomyForm object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\entity_echo\EntityCloner $entity_cloner
   *   The entity cloner service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, EntityCloner $entity_cloner) {
    $this->entityTypeManager = $entity_type_manager;
    $this->entityCloner = $entity_cloner;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('entity_echo.cloner')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'entity_export_taxonomy_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $taxonomy_term = $this->getRequest()->attributes->get('taxonomy_term');

    if (!$taxonomy_term) {
      throw new \InvalidArgumentException('A taxonomy term is required to export.');
    }

    $json = $this->entityCloner->exportEntity($taxonomy_term);

    $form['#title'] = $this->t('Export @label', ['@label' => $taxonomy_term->label()]);

    $form['actions'] = [
      '#type' => 'actions',
      '#weight' => -10,
    ];

    $form['actions']['download'] = [
      '#type' => 'submit',
      '#value' => $this->t('Download JSON'),
      '#button_type' => 'primary',
      '#submit' => ['::downloadSubmit'],
    ];

    $form['actions']['copy'] = [
      '#type' => 'button',
      '#value' => $this->t('Copy JSON'),
      '#attributes' => [
        'class' => ['button', 'button--secondary'],
        'data-copy-target' => 'entity-export-json',
      ],
    ];

    $form['json'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Taxonomy Term json'),
      '#default_value' => $json,
      '#rows' => 20,
      '#attributes' => ['id' => 'entity-export-json'],
      '#description' => $this->t('json representation of the taxonomy term for debugging purposes.'),
      '#weight' => 10,
    ];

    $form['#attached'] = [
      'library' => ['entity_echo/copy'],
      'drupalSettings' => [
        'entityEcho' => [
          'jsonTextareaId' => 'entity-export-json',
        ],
      ],
    ];

    $form_state->set('taxonomy_term', $taxonomy_term);
    return $form;
  }

  /**
   * Submit handler for the download button.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function downloadSubmit(array &$form, FormStateInterface $form_state) {
    $taxonomy_term = $form_state->get('taxonomy_term');
    $json = $form['json']['#default_value'];

    $filename = 'taxonomy_term-' . $taxonomy_term->id() . '.yml';
    $response = new Response($json);
    $response->headers->set('Content-Type', 'text/json');
    $response->headers->set('Content-Disposition', 'attachment; filename="' . $filename . '"');

    $form_state->setResponse($response);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // No action needed for the main submit, as the download button has a
    // custom submit handler.
  }

}
